package com.ai.ipu.cache.config;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.ai.ipu.common.xml.Dom4jHelper;

/**
 * ipu-cache.xml配置文件解析
 *
 * @author huangbo@asiainfo.com
 */
public class IpuCacheConfig {
    private static final String CACHE_GONFIG_FILE = "ipu-cache.xml";
    private static final String ROOT_PATH = "caches";
    private static final String CONFIG_PATH = "cache";
    private static final String CONFIG_PATH_ARRT = CONFIG_PATH + Dom4jHelper.SUFFIX_ATTR;
    
    private static final String SERVERS_PATH = "servers";
    private static final String CONFIG_ATTR = "config_attr";
    private static final String SERVER_ATTR = "server_attr";
    
    private static final String CACHE_NAME = "cache_name";
    private static final String CACHE_TYPE = "cache_type";
    private static final String CACHE_SERVER = "cache_server";
    private static final String CACHE_ATTR = "cache_attr";
    /*属性*/
    private static final String ATTR_NAME = "name";
    private static final String ATTR_VALUE = "value";
    private static final String ATTR_TYPE = "type";
    
    private static IpuCacheConfig config;
    private Map<String, Object> cachesMap;
    
    @SuppressWarnings("unchecked")
    private IpuCacheConfig() throws Exception {
        Dom4jHelper dh = new Dom4jHelper(getClass().getClassLoader().getResourceAsStream(CACHE_GONFIG_FILE));
        Map<String, ?> originData = dh.getAll();
        List<Map<String, Object>> configs = (List<Map<String, Object>>) originData.get(ROOT_PATH);
        cachesMap = new HashMap<String, Object>();
        Map<String, Object> cacheMap;
        List<Map<String, String>> cacheServer;
        Map<String, String> configAttr;
        /*循环缓存实例*/
        for (Map<String, Object> config : configs) {
            cacheMap = new HashMap<String, Object>();
            cacheServer = new ArrayList<Map<String, String>>();
            configAttr = new HashMap<String, String>();
            
            Map<String, String> cacheAttrMap = (Map<String, String>) config.get(CONFIG_PATH_ARRT);
            List<Map<String, Object>> cacheList = (List<Map<String, Object>>)config.get(CONFIG_PATH);
            
            for(Map<String, Object> map : cacheList){
                if(map.get(CONFIG_ATTR)!=null){
                    /*解析缓存实例的属性*/
                    Map<String, String> configValue = (Map<String, String>)map.get(CONFIG_ATTR);
                    configAttr.put(configValue.get(ATTR_NAME), configValue.get(ATTR_VALUE));
                }else if(map.get(SERVERS_PATH)!=null){
                    /*解析缓存实例的服务列表*/
                    List<Map<String, Object>> serverList = (List<Map<String, Object>>)map.get(SERVERS_PATH);
                    for(Map<String, Object> server : serverList){
                        cacheServer.add((Map<String, String>)server.get(SERVER_ATTR));
                    }
                }
            }
            
            cacheMap.put(CACHE_NAME, cacheAttrMap.get(ATTR_NAME));
            cacheMap.put(CACHE_TYPE, cacheAttrMap.get(ATTR_TYPE));
            cacheMap.put(CACHE_ATTR, configAttr);
            cacheMap.put(CACHE_SERVER, cacheServer);
            cachesMap.put(cacheAttrMap.get(ATTR_NAME), cacheMap);
        }
    }

    /**
     * 获取实例
     * @return 返回的config实例
     * @throws Exception
     */
    protected static IpuCacheConfig getInstance() throws Exception {
        if (config == null) {
            config = new IpuCacheConfig();
        }
        return config;
    }

    /**
     * 获取cache map
     * @return 返回cache map 配置关系
     */
    public Map<String, Object> getCacheMap() {
        return cachesMap;
    }

    /**
     * 获取缓存的entity
     * @param cacheName
     * @return 返回缓存的entity
     * @throws Exception
     */
    @SuppressWarnings("unchecked")
    public static Map<String, Object> getCacheEntity(String cacheName) throws Exception {
        // TODO Auto-generated method stub
        return (Map<String, Object>)getInstance().getCacheMap().get(cacheName);
    }

    /**
     * 获取缓存的server
     * @param cacheName
     * @return 返回cache server
     * @throws Exception
     */
    @SuppressWarnings("unchecked")
    public static List<Map<String,String>> getCacheServers(String cacheName) throws Exception {
        // TODO Auto-generated method stub
        return (List<Map<String, String>>) getCacheEntity(cacheName).get(CACHE_SERVER);
    }

    /**
     * 获取缓存的属性
     * @param cacheName
     * @param name
     * @return 返回缓存的属性
     * @throws Exception
     */
    @SuppressWarnings("unchecked")
    public static String getCacheAttr(String cacheName, String name) throws Exception {
        // TODO Auto-generated method stub
        return ((Map<String, String>)getCacheEntity(cacheName).get(CACHE_ATTR)).get(name);
    }

    /**
     * 获取缓存的属性，不存在则使用默认值
     * @param cacheName
     * @param name
     * @param defaultValue
     * @return 缓存的属性
     * @throws Exception
     */
    public static String getCacheDefaultAttr(String cacheName, String name, String defaultValue) throws Exception {
        // TODO Auto-generated method stub
        String value = getCacheAttr(cacheName, name);
        return value == null ? defaultValue : value;
    }

    /**
     * 获取缓存类型
     * @param cacheName
     * @return 缓存的类型
     * @throws Exception
     */
    @SuppressWarnings("unchecked")
    public static String getCacheType(String cacheName) throws Exception {
        // TODO Auto-generated method stub
    	Object cache = getInstance().getCacheMap().get(cacheName);
    	if(cache==null){
    		return null;
    	}
        return (String)((Map<String, Object>)cache).get(CACHE_TYPE);
    }
}
