package com.ai.ipu.cache.mem;

import com.ai.ipu.cache.ICache;

/**
 * MemCache接口
 */
public interface IMemCache extends ICache {
	/**
	 * 在缓存中存放一个K-V键值对，同名键会被覆盖。设置多少秒后超时
	 * @return 是否put成功
	 */
	public boolean put(Object key, Object value, int secondTimeout) throws Exception;
	
	/**
	 * 递增计数器（线程安全），步长为1。
     * @return 增加之后的值
	 */
	public long incr(String cacheKey);
	
	/**
	 * 递增计数器（线程安全），步长为inc。
     * @return 增加之后的值
	 */
	public long incr(String cacheKey, int inc);
	
	/**
	 * 递减计数器（线程安全）， 步长为1。
     * @return 减少之后的值
	 */
	public long decr(String cacheKey);
	
	/**
	 * 递减计数器（线程安全）， 步长为inc。
     * @return 减少之后的值
	 */
	public long decr(String cacheKey, int inc);
}
