package com.ai.ipu.cache;

import com.ai.ipu.basic.log.ILogger;
import com.ai.ipu.basic.log.IpuLoggerFactory;
import com.ai.ipu.cache.CacheFactory.CacheType;

import junit.framework.TestCase;

public class CacheFactoryTest extends TestCase{
    private static final transient ILogger LOGGER = IpuLoggerFactory.createLogger(CacheFactoryTest.class);
    private static final String STR_VALUE = "12345";
    private static final String STR_KEY = "key";
    private static final String CACHE_NAME = "data";
    private static final String CLUSTER_CACHE_NAME = "ssn";
    
    public void testGetCache() throws Exception {
        ICache cache = CacheFactory.getCache(CACHE_NAME);
        LOGGER.debug((String)cache.get(STR_KEY));
        cache.put(STR_KEY, STR_VALUE);
        LOGGER.debug((String)cache.get(STR_KEY));
        assertEquals(STR_VALUE, cache.get(STR_KEY));
        cache.remove(STR_KEY);
        
        cache = CacheFactory.getCache(CacheType.redis, CACHE_NAME);
        LOGGER.debug((String)cache.get(STR_KEY));
        cache.put(STR_KEY, STR_VALUE);
        LOGGER.debug((String)cache.get(STR_KEY));
        assertEquals(STR_VALUE, cache.get(STR_KEY));
        cache.remove(STR_KEY);
        
        CacheFactory.close(CACHE_NAME, cache);
        
        cache = CacheFactory.getCache(CacheType.redis, CACHE_NAME);
        LOGGER.debug((String)cache.get(STR_KEY));
        cache.put(STR_KEY, STR_VALUE);
        LOGGER.debug((String)cache.get(STR_KEY));
        assertEquals(STR_VALUE, cache.get(STR_KEY));
        cache.remove(STR_KEY);
        CacheFactory.close(CACHE_NAME, cache);
    }
    
    public void testAuthFailed() {
        try {
            CacheFactory.getCache(CACHE_NAME);
            assertFalse("校验成功", true);
        } catch (Exception e) {
            LOGGER.debug(e.getMessage());
            assertEquals("NOAUTH Authentication required.", e.getMessage());;
        }
    }
    
    public void testGetClusterCache() throws Exception {
        ICache cache = CacheFactory.getCache(CLUSTER_CACHE_NAME);
        LOGGER.debug((String)cache.get(STR_KEY));
        cache.put(STR_KEY, STR_VALUE);
        LOGGER.debug((String)cache.get(STR_KEY));
        assertEquals(STR_VALUE, cache.get(STR_KEY));
        cache.remove(STR_KEY);
        
        cache = CacheFactory.getCache(CacheType.redis, CLUSTER_CACHE_NAME);
        LOGGER.debug((String)cache.get(STR_KEY));
        cache.put(STR_KEY, STR_VALUE);
        LOGGER.debug((String)cache.get(STR_KEY));
        assertEquals(STR_VALUE, cache.get(STR_KEY));
        
    }
}
