package com.ai.ipu.cache.redis;

import java.io.IOException;
import java.util.HashSet;
import java.util.Set;

import junit.framework.TestCase;
import redis.clients.jedis.HostAndPort;
import redis.clients.jedis.JedisCluster;
import redis.clients.jedis.JedisPoolConfig;

import com.ai.ipu.basic.log.ILogger;
import com.ai.ipu.basic.log.IpuLoggerFactory;
import com.ai.ipu.cache.CacheFactory;
import com.ai.ipu.cache.CacheFactory.CacheType;
import com.ai.ipu.cache.ICache;

/**
 * @author huangbo@asiainfo.com
 * @team IPU
 * @date 2017年11月6日下午4:43:03
 * @desc Jedis原始包集群操作使用范例
 */
public class RedisClusterTest extends TestCase{
	private static final transient ILogger LOGGER = IpuLoggerFactory.createLogger(RedisClusterTest.class);
    private String[] hosts = {
                             "121.42.183.206",
                             "121.42.183.206",
                             "121.42.183.206",
                             "121.42.183.206",
                             "121.42.183.206",
                             "121.42.183.206"
                             };
    private int[] ports = {7101,7102,7103,7104,7105,7106};
    
    @Override
    protected void setUp() throws Exception {
        // TODO Auto-generated method stub
        super.setUp();
    }
    
    public void testCluster() throws Exception{
        JedisCluster jedisCluster = RedisCacheFactory.getJedisCluster("data");
        try {
            long start = System.currentTimeMillis();
            for (int i = 0; i < 100; i++) {
                @SuppressWarnings("unused")
                String result = jedisCluster.set("x" + i, "n" + i);
            }
            long end = System.currentTimeMillis();
            LOGGER.debug("普通同步调用方式耗时: " + ((end - start) / 1000.0) + "秒");
            jedisCluster.close();
        }catch (IOException e) {
        	LOGGER.error("IOException", e);
        }
    }
    
    public void testClusterWithAuth() throws Exception{
        JedisCluster jedisCluster = RedisCacheFactory.getJedisCluster("ssn");
        
        try {
            long start = System.currentTimeMillis();
            for (int i = 0; i < 100; i++) {
                jedisCluster.set("d" + i, "d" + i);
            }
            long end = System.currentTimeMillis();
            LOGGER.debug("普通同步调用方式耗时: " + ((end - start) / 1000.0) + "秒");
            jedisCluster.close();
        }catch (IOException e) {
        	LOGGER.error("IOException", e);
        }
    }
    
    public void testClusterPipelineWithAuth(){
        //GenericObjectPoolConfig的使用
        JedisPoolConfig config = new JedisPoolConfig();
        Set<HostAndPort> nodes = new HashSet<HostAndPort>();
        for(int i=0,len=hosts.length;i<len;i++){
            nodes.add(new HostAndPort(hosts[i], ports[i]));
        }
        JedisCluster jedisCluster = new JedisCluster(nodes, 5000, 5000, 3, "ipu",  config);
        try {
            long start = System.currentTimeMillis();
            for (int i = 0; i < 100; i++) {
                jedisCluster.set("b" + i, "b" + i);
            }
            long end = System.currentTimeMillis();
            LOGGER.debug("普通同步调用方式耗时: " + ((end - start) / 1000.0) + "秒");
            jedisCluster.close();
        }catch (IOException e) {
        	LOGGER.error("IOException", e);
        }
    }
    
    
    public void testCacheFactory()
    {
        try {
			ICache cache = CacheFactory.getCache(CacheType.redis, "jedisCluster");
			boolean result = cache.put("a", "a");
			
			LOGGER.debug(Boolean.toString(result));
			LOGGER.debug((String)cache.get("a"));
			LOGGER.debug(Boolean.toString(cache.remove("a")));
			LOGGER.debug(Boolean.toString(cache.remove("1")));
			
		} catch (Exception e) {
			LOGGER.error("IOException", e);
		}
    }
}
